"""     
###################################################################################
#                                                                                 #
#   Copyright "2015" "NEXT ENERGY"                                                #
#                                                                                 #
#   Licensed under the Apache License, Version 2.0 (the "License");               #
#   you may not use this file except in compliance with the License.              #
#   You may obtain a copy of the License at                                       #
#                                                                                 #
#       http://www.apache.org/licenses/LICENSE-2.0                                #
#                                                                                 #
#   Unless required by applicable law or agreed to in writing, software           #
#   distributed under the License is distributed on an "AS IS" BASIS,             #
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.      #
#   See the License for the specific language governing permissions and           #
#   limitations under the License.                                                #
#                                                                                 #
###################################################################################
"""
# Command line usage:
# python SciGRID.py -U db_user -P db_port -D db_name -H db_host
#
# Python object usage:
# SciGRID_instance = SciGRID(database='db_name', user='db_user', \
# port='db_port', password='your_password', host='db_host')

from __future__ import absolute_import
import os
import logging
logging = logging.getLogger(os.path.basename(__file__))
from db_common import dbconn_from_args
from create_db_tables import create_tables
from relation_analysis import analyze_rels
import relation_abstraction as ra
import electrical_properties as ep
from create_plots import create_de_topology_plot

class SciGRID:
    """ 
    SciGRID class provides methods for the abstraction of 
    OpenStreetMap power data. 
    """

    def __init__(self, dbconn):
        # Initializes the SciGRID class
        self.conn = dbconn
        self.cur  = dbconn.cursor()

    def create_tables(self):
	# Create tables necessary for the abstraction process. 
        create_tables(self.cur, self.conn)
    
    def analyze_relations(self):
	# Analyze the power relations. 
        msg = analyze_rels(self.cur, self.conn)
        return msg
    
    def abstract_relations(self):
	# Execute the abstraction. 
        msg = ra.abstract_2subs(self.cur, self.conn)
        success = ra.abstract_3subs_T_node(self.cur, self.conn)
        #TODO: ra.abstract_3subs_in_row(self.cur, self.conn)
        return len(msg) + success
        
    def add_electrical_properties(self):
	# Calculate the electrical properties of the transmission lines using the dena assumptions. 
        nr_success = ep.add_dena_assumptions(self.cur, self.conn)
        return nr_success
    
    def create_topology_plot(self):
        # Create a plot of the resulting abstracted (topological) transmission network. 
        # For the moment it is just a plot for the German network.
        plot = create_de_topology_plot(self.cur,self.conn)
        return (plot)
    
if __name__ == '__main__':
    # Connect to DB
    try:
        dbconn = dbconn_from_args()
    except Exception, e:
        logging.error('Could not connect to database. Please check the values of host,port,user,password, and database name.', exc_info=True)
        exit(1)
    
    # Excute SciGRID methods
    try:
        SciGRID_instance = SciGRID(dbconn)
        SciGRID_instance.create_tables()
        logging.info('Tables created.')
        msg = SciGRID_instance.analyze_relations()
        logging.info('Number of analyzed relations: ' + str(len(msg)))
        nr_success = SciGRID_instance.abstract_relations()
        logging.info('Number of abstracted relations: ' + str(nr_success))
        nr_success = SciGRID_instance.add_electrical_properties() 
        logging.info('Calculated electrical properties: ' + str(nr_success))
        plots = SciGRID_instance.create_topology_plot()
        # FIXME: return message for plots
        logging.info('Plotted: ' + str(plots))
        SciGRID_instance.cur.close()
        SciGRID_instance.conn.close()
        logging.info('All done. Finished the execution of SciGRID methods')
    except Exception, e:
        SciGRID_instance.cur.close()
        SciGRID_instance.conn.close()
        logging.error('Could not perform SciGRID abstraction.', exc_info=True)
        exit(1)
    
  

    

    
    
