"""     
###################################################################################
#                                                                                 #
#   Copyright "2015" "NEXT ENERGY"                                                #
#                                                                                 #
#   Licensed under the Apache License, Version 2.0 (the "License");               #
#   you may not use this file except in compliance with the License.              #
#   You may obtain a copy of the License at                                       #
#                                                                                 #
#       http://www.apache.org/licenses/LICENSE-2.0                                #
#                                                                                 #
#   Unless required by applicable law or agreed to in writing, software           #
#   distributed under the License is distributed on an "AS IS" BASIS,             #
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.      #
#   See the License for the specific language governing permissions and           #
#   limitations under the License.                                                #
#                                                                                 #
###################################################################################
"""
import os
import logging
logging = logging.getLogger(os.path.basename(__file__))
from db_common import dbconn_from_args
import sys
import os

def ct__analysis_rels(cur,conn):
    # Create _analysis_rels table.
    sql = """
          DROP TABLE IF EXISTS _analysis_rels;
          CREATE TABLE _analysis_rels (
          id                  serial PRIMARY KEY NOT NULL, 
          osm_id              bigint, 
          voltage             integer,
          cables              smallint,
          wires               text,
          wires_nb            smallint,
          frequency           text,
          num_parts           smallint,
          num_stations        smallint,
          num_transmissions   smallint,
          num_discardeds      smallint,
          stations            text[],
          transmissions       text[],
          discardeds          text[],
          T_node_ids          bigint[],
          incomplete          text,
          abstracted          smallint);
          """
    cur.execute(sql)
    conn.commit()


def ct__vertices(cur,conn):
    # Create _vertices table.
    sql = """
          DROP TABLE IF EXISTS _vertices;
          CREATE TABLE _vertices (
          id              serial PRIMARY KEY NOT NULL, 
          osm_id          bigint,
          osm_id_typ      char, 
          geo_center      geometry,
          longitude       float,
          latitude        float,
          role            text,
          voltage         text,
          from_relation   bigint);
          """
    cur.execute(sql)
    conn.commit()


def ct__links(cur,conn):
    # Create _links table.
    sql = """
          DROP TABLE IF EXISTS _links;
          CREATE TABLE _links (
          id                 serial PRIMARY KEY NOT NULL, 
          osm_id_1           bigint, 
          osm_id_1_typ       char,
          osm_id_2           bigint,
          osm_id_2_typ       char,
          length_m           integer,
          way                geometry,
          voltage            integer,
          cables             integer,
          wires              text,
          wires_nb           integer,
          frequency          text,
          from_relation      bigint,
          from_transmissions bigint[]);
          """
    cur.execute(sql)
    conn.commit()


def ct__t_node_problems(cur,conn):
    # Create _t_node_problems table.
    sql = """
          DROP TABLE IF EXISTS _t_node_problems;
          CREATE TABLE _t_node_problems ( 
          id              serial PRIMARY KEY NOT NULL, 
          rel_id          bigint, 
          T_node_ids      bigint[], 
          stations        text[], 
          transmissions   text[],
          error_msg       text);
          """
    cur.execute(sql)
    conn.commit()


def ct__msg_log(cur,conn):
    # Create msg_log table.
    sql = """
          DROP TABLE IF EXISTS msg_log;
          CREATE TABLE msg_log ( 
          id             serial PRIMARY KEY NOT NULL, 
          function       text, 
          typ            text, 
          message        text);
          """
    cur.execute(sql)
    conn.commit()


def ct_vertices_ref_id(cur,conn):
    # Create _vertices_ref_id table.
    sql = """
          CREATE TABLE IF NOT EXISTS vertices_ref_id (
          v_id            serial PRIMARY KEY NOT NULL,
          osm_id          bigint,
          osm_id_typ      char,
          visible         smallint);        
          """
    cur.execute(sql)
    conn.commit()



def ct_vertices(cur,conn):
    # Create vertices table.
    sql = """
          DROP TABLE IF EXISTS vertices;
          CREATE TABLE vertices (
          v_id             bigint PRIMARY KEY NOT NULL,
          lon              float,
          lat              float,
          typ              text,
          voltage          text,
          frequency        text,
          name             text,
          operator         text,
          ref              text,
          WKT_SRID_4326    text);
          """
    cur.execute(sql)
    conn.commit()


def ct_links(cur,conn):
    # Create links table.
    sql = """
          DROP TABLE IF EXISTS links;
          CREATE TABLE links (
          l_id             serial PRIMARY KEY NOT NULL,
          v_id_1           bigint NOT NULL,
          v_id_2           bigint NOT NULL,
          voltage          integer,
          cables           integer,
          wires            integer,
          frequency        text,
          name             text,
          operator         text,
          ref              text,
          length_m         integer,
          r_ohmKm          float,
          x_ohmKm          float,
          c_nFKm           float,
          i_th_max_A       float,
          from_relation    bigint,
          WKT_SRID_4326    text);
          """
    cur.execute(sql)
    conn.commit()


def ct_wires_ref_numbers(cur,conn):
    # Create links table.
    sql = """
          DROP TABLE IF EXISTS wires_ref_numbers;
          CREATE TABLE wires_ref_numbers (
          name             TEXT,
          value            INTEGER);
          INSERT INTO wires_ref_numbers (name,value) VALUES
          ('single',1), ('si', 1), ('sinlge',1), ('1',1), 
          ('double', 2), ('2', 2), 
          ('triple', 3), ('3', 3), 
          ('quad', 4), ('quaed', 4), ('quead', 4), ('4', 4), 
          ('fivefold', 5),
          ('sixfold', 6), ('6', 6), 
          ('eightfold', 8);
          """
    cur.execute(sql)
    conn.commit()

def ct_vertices_ref_names(cur,conn):
    # Create links table.
    # QUERRY to get exceptions:
    # SELECT DISTINCT unnest(string_to_array(hstore(tags)->'power',';')) FROM planet_osm_rels 
    # UNION 
    # SELECT DISTINCT unnest(string_to_array(hstore(tags)->'power',';')) FROM planet_osm_ways 
    # UNION SELECT DISTINCT unnest(string_to_array(hstore(tags)->'power',';')) FROM planet_osm_nodes
    sql = """
          DROP TABLE IF EXISTS vertices_ref_names;
          CREATE TABLE vertices_ref_names (
          name             TEXT,
          value            TEXT);
          INSERT INTO vertices_ref_names (name,value) VALUES
          ('substation','substation'), ('sub_station', 'substation'), ('station','substation'),
          ('transformer','transformer'), ('transfrmer','transformer'), 
          ('generator','generator'),
          ('plant','plant');
          """
    cur.execute(sql)
    conn.commit()

 
def create_tables(cur,conn):
    # Create all tables
    ct__analysis_rels(cur,conn)
    ct__vertices(cur,conn)
    ct__links(cur,conn)
    ct__t_node_problems(cur,conn)
    ct__msg_log(cur,conn)
    ct_vertices_ref_id(cur,conn)
    ct_vertices(cur,conn)
    ct_links(cur,conn)
    ct_wires_ref_numbers(cur,conn)
    ct_vertices_ref_names(cur,conn)
    
    
if __name__ == '__main__':
    try:
        conn       = dbconn_from_args()
        cur        = conn.cursor()
        create_tables(cur,conn)
        logging.info('Tables created.')
    except Exception, e:
        logging.error('Could not create tables in database.', exc_info=True)
        exit(1)
