"""     
###################################################################################
#                                                                                 #
#   Copyright "2015" "NEXT ENERGY"                                                #
#                                                                                 #
#   Licensed under the Apache License, Version 2.0 (the "License");               #
#   you may not use this file except in compliance with the License.              #
#   You may obtain a copy of the License at                                       #
#                                                                                 #
#       http://www.apache.org/licenses/LICENSE-2.0                                #
#                                                                                 #
#   Unless required by applicable law or agreed to in writing, software           #
#   distributed under the License is distributed on an "AS IS" BASIS,             #
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.      #
#   See the License for the specific language governing permissions and           #
#   limitations under the License.                                                #
#                                                                                 #
###################################################################################
"""
import os
import logging
logging = logging.getLogger(os.path.basename(__file__))
from db_common import dbconn_from_args
import matplotlib.pyplot as plt
from matplotlib import cm
cmap = cm.spectral


def plot_landmass_germany(fig):
    """ Plot landmass of Germany from a de_landmass.txt file which is an border extraction of relation 62781 from OpenStreetMap. """
    de = open('../data/04_visualization/de_landmass.txt','r')
    max_lat = 0.0
    max_lon = 0.0
    min_lat = 360.0
    min_lon = 360.0
    for line in de:
        line = line.replace('\n','')
        (lon,lat) = line.split('|')
        plt.plot(lon.split(','), lat.split(','), color = 'black', lw = 0.3, zorder = 1)
        if max_lon < float(max(lon.split(','))):
            max_lon = float(max(lon.split(',')))
        if max_lat < float(max(lat.split(','))):
            max_lat = float(max(lat.split(',')))
        if min_lon > float(min(lon.split(','))):
            min_lon = float(min(lon.split(',')))
        if min_lat > float(min(lat.split(','))):
            min_lat = float(min(lat.split(',')))
    
    fig.set_size_inches(1.3 * (float(max_lat)-float(min_lat)), 1.3 * (float(max_lon)-float(min_lon)))
    return fig

 
def plot_topology(cur,conn,fig):
    """ Plot the topological transmission network from the vertices and links database."""
    # Query to obtain the network vertices.
    sql = 'SELECT v_id, lon, lat FROM vertices ORDER BY v_id;'
    cur.execute(sql)
    res = cur.fetchall()
    vertices = {v_id : [lon,lat] for (v_id, lon, lat) in res} 
    plt.plot(*zip(*vertices.values()), marker='o', markerfacecolor='black', linestyle="None", markersize=3, zorder=2)
    
    # Query to obtain the network links.
    sql = 'SELECT v_id_1, v_id_2, voltage FROM links ORDER BY voltage'
    cur.execute(sql)
    links = cur.fetchall()
    
    occured_voltages = set()
    for (v_id_1,v_id_2,voltage) in links:
        start_of_link = vertices[v_id_1]
        end_of_link = vertices[v_id_2]
        if voltage >= 380000 and voltage <= 400000:
            voltage = 390000            
        occured_voltages.add(voltage)
        plt.plot(*zip(start_of_link,end_of_link), color = cmap(int(255 * ((voltage - 110000) / 340000.0))), lw = 1.3, zorder = 1)
    
    plt.plot([], [], marker='o', markerfacecolor='black', linestyle="None", markersize=3, zorder=2, label = 'substation')
    for voltage in sorted(occured_voltages):
        if voltage >= 380000 and voltage <= 400000:
            plt.plot([], [], color = cmap(int(255 * ( (390000 - 110000) / 340000.0))), lw = 1.3, zorder = 2, label = "380-400 kV transmission line")    
        else:
            plt.plot([], [], color = cmap(int(255 * ( (voltage - 110000) / 340000.0))), lw = 1.3, zorder = 2, label = str(int(voltage / 1000.0)) + " kV transmission line")
    l = plt.legend(numpoints=1,loc=1)
    l.set_zorder(3)
    return fig


def create_de_topology_plot(cur,conn):
    fig = plt.figure(figsize=(10,12),facecolor = 'white')
    ax = plt.subplot(111)
    ax.set_axis_off()
    fig.add_axes(ax)    
    
    fig = plot_landmass_germany(fig)
    fig = plot_topology(cur,conn,fig)
    
    destination = '../data/04_visualization/'  
    sql = "SELECT current_database();"
    cur.execute(sql)    
    dbname = cur.fetchone()[0]
    filename = destination + 'topology_' + dbname + '.png'
    plt.savefig(filename, bbox_inches='tight', pad_inches=0, dpi=600)
    return filename

if __name__ == '__main__':
    try:
        conn       = dbconn_from_args()
        cur        = conn.cursor()
        plots = create_de_topology_plot(cur,conn)
        logging.info('Plotted: ' + str(plots))
    except Exception, e:
        logging.error('Could not create plot of the network topology.', exc_info=True)
        exit(1)
    
